//
// SPDX-License-Identifier: BSD-3-Clause
// Copyright (c) Contributors to the OpenEXR Project.
//

#include "ImfNamespace.h"
#include "ImfCompression.h"

bool equivalent (float f1, float f2, OPENEXR_IMF_NAMESPACE::Compression comp);

