//
// SPDX-License-Identifier: BSD-3-Clause
// Copyright (c) Contributors to the OpenEXR Project.
//

#ifndef TESTMULTITILEDPARTTHREADING_H_
#define TESTMULTITILEDPARTTHREADING_H_

#include <string>

void testMultiTiledPartThreading (const std::string & tempDir);

#endif /* TESTMULTITILEDPARTTHREADING_H_ */
