//
// SPDX-License-Identifier: BSD-3-Clause
// Copyright (c) Contributors to the OpenEXR Project.
//


#ifndef TESTBACKWARDCOMPATIBILITY_H_
#define TESTBACKWARDCOMPATIBILITY_H_

#include <string>

void testBackwardCompatibility (const std::string & tempDir);

#endif /* TESTBACKWARDCOMPATIBILITY_H_ */
